@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: getdef.c,v 1.40 2006/06/24 13:17:18 kloczek Exp $"
+#ident "$Id: getdef.c 1442 2007-11-23 00:07:59Z nekral-guest $"
 
 #include "prototypes.h"
 #include "defines.h"
@@ -52,6 +52,7 @@ static struct itemdef def_table[] = {
 	{"CONSOLE", NULL},
 	{"CREATE_HOME", NULL},
 	{"DEFAULT_HOME", NULL},
+	{"ENCRYPT_METHOD", NULL},
 	{"ENV_PATH", NULL},
 	{"ENV_SUPATH", NULL},
 	{"ERASECHAR", NULL},
@@ -67,9 +68,13 @@ static struct itemdef def_table[] = {
 	{"LOG_UNKFAIL_ENAB", NULL},
 	{"MAIL_DIR", NULL},
 	{"MAIL_FILE", NULL},
+	{"MAX_MEMBERS_PER_GROUP", NULL},
+	{"MD5_CRYPT_ENAB", NULL},
 	{"PASS_MAX_DAYS", NULL},
 	{"PASS_MIN_DAYS", NULL},
 	{"PASS_WARN_AGE", NULL},
+	{"SHA_CRYPT_MAX_ROUNDS", NULL},
+	{"SHA_CRYPT_MIN_ROUNDS", NULL},
 	{"SULOG_FILE", NULL},
 	{"SU_NAME", NULL},
 	{"TTYGROUP", NULL},
@@ -93,7 +98,6 @@ static struct itemdef def_table[] = {
 	{"LASTLOG_ENAB", NULL},
 	{"LOGIN_STRING", NULL},
 	{"MAIL_CHECK_ENAB", NULL},
-	{"MD5_CRYPT_ENAB", NULL},
 	{"MOTD_FILE", NULL},
 	{"NOLOGINS_FILE", NULL},
 	{"OBSCURE_CHECKS_ENAB", NULL},
