@@ -6,7 +6,7 @@
  * Juha Virtanen, <jiivee@hut.fi>; November 1995
  */
 /*
- * $Id: prototypes.h,v 1.28 2006/02/07 16:36:30 kloczek Exp $
+ * $Id: prototypes.h 1448 2007-11-23 20:11:00Z nekral-guest $
  *
  * Added a macro to work around ancient (non-ANSI) compilers, just in case
  * someone ever tries to compile this with SunOS cc...  --marekm
@@ -23,6 +23,7 @@
 #endif
 #include <pwd.h>
 #include <grp.h>
+#include "shadowio.h"
 
 #include "defines.h"
 
@@ -69,6 +70,9 @@ extern void sanitize_env (void);
 extern void change_field (char *, size_t, const char *);
 extern int valid_field (const char *, const char *);
 
+/* getlong.c */
+extern int getlong(const char *numstr, long int *result);
+
 /* fputsx.c */
 extern char *fgetsx (char *, int, FILE *);
 extern int fputsx (const char *, FILE *);
@@ -131,7 +135,7 @@ extern void pwd_init (void);
 extern int do_rlogin (const char *, char *, int, char *, int);
 
 /* salt.c */
-extern char *crypt_make_salt (void);
+extern char *crypt_make_salt (char *meth, void *arg);
 
 /* setugid.c */
 extern int setup_groups (const struct passwd *);
@@ -179,4 +183,15 @@ extern int valid (const char *, const struct passwd *);
 extern char *xmalloc (size_t);
 extern char *xstrdup (const char *);
 
+/* xgetpwnam.c */
+extern struct passwd *xgetpwnam (const char *);
+/* xgetpwuid.c */
+extern struct passwd *xgetpwuid (uid_t);
+/* xgetgrnam.c */
+extern struct group *xgetgrnam (const char *);
+/* xgetgrgid.c */
+extern struct group *xgetgrgid (gid_t);
+/* xgetspnam.c */
+extern struct spwd *xgetspnam(const char *);
+
 #endif				/* _PROTOTYPES_H */
