@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: chsh.c,v 1.42 2006/07/13 21:26:35 kloczek Exp $"
+#ident "$Id: chsh.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <fcntl.h>
 #include <getopt.h>
@@ -80,8 +80,8 @@ static void usage (void)
 	fprintf (stderr, _("Usage: chsh [options] [LOGIN]\n"
 			   "\n"
 			   "Options:\n"
-			   "  -h, --help				display this help message and exit\n"
-			   "  -s, --shell SHELL			new login shell for the user account\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -s, --shell SHELL             new login shell for the user account\n"
 			   "\n"));
 	exit (E_USAGE);
 }
@@ -182,7 +182,6 @@ int main (int argc, char **argv)
 
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -248,7 +247,7 @@ int main (int argc, char **argv)
 	 */
 	if (optind < argc) {
 		user = argv[optind];
-		pw = getpwnam (user);
+		pw = xgetpwnam (user);
 		if (!pw) {
 			fprintf (stderr,
 				 _("%s: unknown user %s\n"), Prog, user);
@@ -341,13 +340,17 @@ int main (int argc, char **argv)
 #else				/* !USE_PAM */
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("chsh", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("chsh", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
