@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: expiry.c,v 1.19 2006/02/08 10:53:16 kloczek Exp $"
+#ident "$Id: expiry.c 1397 2007-11-18 23:15:26Z nekral-guest $"
 
 #include <pwd.h>
 #include <signal.h>
@@ -61,7 +61,7 @@ static void usage (void)
 /* 
  * expiry - check and enforce password expiration policy
  *
- *	expiry checks (-c) the current password expiraction and forces (-f)
+ *	expiry checks (-c) the current password expiration and forces (-f)
  *	changes when required. It is callable as a normal user command.
  */
 int main (int argc, char **argv)
@@ -101,7 +101,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, _("%s: unknown user\n"), Prog);
 		exit (10);
 	}
-	spwd = getspnam (pwd->pw_name);
+	spwd = getspnam (pwd->pw_name); /* !USE_PAM, No need for xgetspnam */
 
 	/*
 	 * If checking accounts, use agecheck() function.
