@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: gpasswd.c,v 1.36 2006/02/08 10:58:46 kloczek Exp $"
+#ident "$Id: gpasswd.c 1441 2007-11-22 21:55:12Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -121,7 +121,7 @@ static int check_list (const char *users)
 	const char *start, *end;
 	char username[32];
 	int errors = 0;
-	int len;
+	size_t len;
 
 	for (start = users; start && *start; start = end) {
 		if ((end = strchr (start, ','))) {
@@ -140,7 +140,7 @@ static int check_list (const char *users)
 		 * This user must exist.
 		 */
 
-		if (!getpwnam (username)) {
+		if (!getpwnam (username)) { /* local, no need for xgetpwnam */
 			fprintf (stderr, _("%s: unknown user %s\n"),
 				 Prog, username);
 			errors++;
@@ -171,12 +171,12 @@ int main (int argc, char **argv)
 	char *cp;
 	int amroot;
 	int retries;
-	struct group *gr = NULL;
+	struct group const*gr = NULL;
 	struct group grent;
 	static char pass[BUFSIZ];
 
 #ifdef	SHADOWGRP
-	struct sgrp *sg = NULL;
+	struct sgrp const*sg = NULL;
 	struct sgrp sgent;
 	char *admins = NULL;
 #endif
@@ -217,6 +217,7 @@ int main (int argc, char **argv)
 		switch (flag) {
 		case 'a':	/* add a user */
 			user = optarg;
+			/* local, no need for xgetpwnam */
 			if (!getpwnam (user)) {
 				fprintf (stderr,
 					 _("%s: unknown user %s\n"), Prog,
@@ -313,14 +314,20 @@ int main (int argc, char **argv)
 	 * will be completely replicated so it may be modified later on.
 	 */
 
-	/*
-	 * XXX - should get the entry using gr_locate() and modify that,
-	 * getgrnam() could give us a NIS group.  --marekm
-	 */
 	if (!(group = argv[optind]))
 		usage ();
 
-	if (!(gr = getgrnam (group))) {
+	if (!gr_open (O_RDONLY)) {
+		fprintf (stderr, _("%s: can't open file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "opening /etc/group",
+			      group, -1, 0);
+#endif
+		exit (1);
+	}
+
+	if (!(gr = gr_locate (group))) {
 		fprintf (stderr, _("unknown group: %s\n"), group);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "group lookup", group,
@@ -333,8 +340,26 @@ int main (int argc, char **argv)
 	grent.gr_passwd = xstrdup (gr->gr_passwd);
 
 	grent.gr_mem = dup_list (gr->gr_mem);
+	if (!gr_close ()) {
+		fprintf (stderr, _("%s: can't close file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot close /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "closing /etc/group", group, -1, 0);
+#endif
+		exit (1);
+	}
 #ifdef	SHADOWGRP
-	if ((sg = getsgnam (group))) {
+	if (!sgr_open (O_RDONLY)) {
+		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "opening /etc/gshadow", group, -1, 0);
+#endif
+		exit (1);
+	}
+	if ((sg = sgr_locate (group))) {
 		sgent = *sg;
 		sgent.sg_name = xstrdup (sg->sg_name);
 		sgent.sg_passwd = xstrdup (sg->sg_passwd);
@@ -359,6 +384,15 @@ int main (int argc, char **argv)
 
 		sg = &sgent;
 	}
+	if (!sgr_close ()) {
+		fprintf (stderr, _("%s: can't close shadow file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot close /etc/gshadow"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "closing /etc/gshadow", group, -1, 0);
+#endif
+		exit (1);
+	}
 
 	/*
 	 * The policy here for changing a group is that 1) you must be root
@@ -606,7 +640,7 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
-	cp = pw_encrypt (pass, crypt_make_salt ());
+	cp = pw_encrypt (pass, crypt_make_salt (NULL, NULL));
 	memzero (pass, sizeof pass);
 #ifdef SHADOWGRP
 	if (is_shadowgrp)
