@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: groupadd.c,v 1.56 2006/08/15 17:25:58 kloczek Exp $"
+#ident "$Id: groupadd.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <ctype.h>
 #include <fcntl.h>
@@ -98,13 +98,13 @@ static void usage (void)
 	fprintf (stderr, _("Usage: groupadd [options] GROUP\n"
 			   "\n"
 			   "Options:\n"
-			   "  -f, --force			force exit with success status if the specified\n"
-			   "				group already exists\n"
-			   "  -g, --gid GID			use GID for the new group\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -K, --key KEY=VALUE		overrides /etc/login.defs defaults\n"
-			   "  -o, --non-unique		allow create group with duplicate\n"
-			   "				(non-unique) GID\n"
+			   "  -f, --force                   force exit with success status if the\n"
+			   "                                specified group already exists\n"
+			   "  -g, --gid GID                 use GID for the new group\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -K, --key KEY=VALUE           overrides /etc/login.defs defaults\n"
+			   "  -o, --non-unique              allow create group with duplicate\n"
+			   "                                (non-unique) GID\n"
 			   "\n"));
 	exit (E_USAGE);
 }
@@ -259,6 +259,7 @@ static void find_new_gid (void)
 			if (!grp)
 				break;
 #else
+			/* local, no need for xgetgrgid */
 			if (!getgrgid (group_id))
 				break;
 #endif
@@ -395,7 +396,6 @@ int main (int argc, char **argv)
 {
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -490,13 +490,17 @@ int main (int argc, char **argv)
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("groupadd", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("groupadd", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -526,7 +530,7 @@ int main (int argc, char **argv)
 	/*
 	 * Start with a quick check to see if the group exists.
 	 */
-	if (getgrnam (group_name)) {
+	if (getgrnam (group_name)) { /* local, no need for xgetgrnam */
 		if (fflg) {
 			exit (E_SUCCESS);
 		}
