@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: passwd.c,v 1.61 2006/07/28 17:40:15 kloczek Exp $"
+#ident "$Id: passwd.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -96,7 +96,20 @@ static long inact = 0;		/* Days without change before locked */
 static int do_update_age = 0;
 
 #ifndef USE_PAM
-static char crypt_passwd[128];	/* The "old-style" password, if present */
+/*
+ * Size of the biggest passwd:
+ *   $6$	3
+ *   rounds=	7
+ *   999999999	9
+ *   $		1
+ *   salt	16
+ *   $		1
+ *   SHA512	123
+ *   nul	1
+ *
+ *   total	161
+ */
+static char crypt_passwd[256];
 static int do_update_pwd = 0;
 #endif
 
@@ -133,23 +146,23 @@ static void usage (int status)
 	fprintf (stderr, _("Usage: passwd [options] [LOGIN]\n"
 			   "\n"
 			   "Options:\n"
-			   "  -a, --all			report password status on all accounts\n"
-			   "  -d, --delete			delete the password for the named account\n"
-			   "  -e, --expire			force expire the password for the named account\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -k, --keep-tokens		change password only if expired\n"
-			   "  -i, --inactive INACTIVE	set password inactive after expiration\n"
-			   "				to INACTIVE\n"
-			   "  -l, --lock			lock the named account\n"
-			   "  -n, --mindays MIN_DAYS	set minimum number of days before password\n"
-			   "				change to MIN_DAYS\n"
-			   "  -q, --quiet			quiet mode\n"
-			   "  -r, --repository REPOSITORY	change password in REPOSITORY repository\n"
-			   "  -S, --status			report password status on the named account\n"
-			   "  -u, --unlock			unlock the named account\n"
-			   "  -w, --warndays WARN_DAYS	set expiration warning days to WARN_DAYS\n"
-			   "  -x, --maxdays MAX_DAYS	set maximim number of days before password\n"
-			   "				change to MAX_DAYS\n"
+			   "  -a, --all                     report password status on all accounts\n"
+			   "  -d, --delete                  delete the password for the named account\n"
+			   "  -e, --expire                  force expire the password for the named account\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -k, --keep-tokens             change password only if expired\n"
+			   "  -i, --inactive INACTIVE       set password inactive after expiration\n"
+			   "                                to INACTIVE\n"
+			   "  -l, --lock                    lock the named account\n"
+			   "  -n, --mindays MIN_DAYS        set minimum number of days before password\n"
+			   "                                change to MIN_DAYS\n"
+			   "  -q, --quiet                   quiet mode\n"
+			   "  -r, --repository REPOSITORY   change password in REPOSITORY repository\n"
+			   "  -S, --status                  report password status on the named account\n"
+			   "  -u, --unlock                  unlock the named account\n"
+			   "  -w, --warndays WARN_DAYS      set expiration warning days to WARN_DAYS\n"
+			   "  -x, --maxdays MAX_DAYS        set maximim number of days before password\n"
+			   "                                change to MAX_DAYS\n"
 			   "\n"));
 	exit (status);
 }
@@ -190,7 +203,8 @@ static int new_password (const struct passwd *pw)
 	char pass[200];		/* New password */
 	int i;			/* Counter for retries */
 	int warned;
-	int pass_max_len;
+	int pass_max_len = -1;
+	char *method;
 
 #ifdef HAVE_LIBCRACK_HIST
 	int HistUpdate (const char *, const char *);
@@ -228,15 +242,33 @@ static int new_password (const struct passwd *pw)
 	 * for strength, unless it is the root user. This provides an escape
 	 * for initial login passwords.
 	 */
-	if (getdef_bool ("MD5_CRYPT_ENAB"))
-		pass_max_len = 127;
-	else
-		pass_max_len = getdef_num ("PASS_MAX_LEN", 8);
-
-	if (!qflg)
-		printf (_("\
-Enter the new password (minimum of %d, maximum of %d characters)\n\
-Please use a combination of upper and lower case letters and numbers.\n"), getdef_num ("PASS_MIN_LEN", 5), pass_max_len);
+	if ((method = getdef_str ("ENCRYPT_METHOD")) == NULL) {
+		if (!getdef_bool ("MD5_CRYPT_ENAB"))
+			pass_max_len = getdef_num ("PASS_MAX_LEN", 8);
+	} else {
+		if (   !strcmp (method, "MD5")
+#ifdef USE_SHA_CRYPT
+		    || !strcmp (method, "SHA256")
+		    || !strcmp (method, "SHA512")
+#endif
+		    )
+			pass_max_len = -1;
+		else
+			pass_max_len = getdef_num ("PASS_MAX_LEN", 8);
+	}
+	if (!qflg) {
+		if (pass_max_len == -1) {
+			printf (_(
+"Enter the new password (minimum of %d characters)\n"
+"Please use a combination of upper and lower case letters and numbers.\n"),
+				getdef_num ("PASS_MIN_LEN", 5));
+		} else {
+			printf (_(
+"Enter the new password (minimum of %d, maximum of %d characters)\n"
+"Please use a combination of upper and lower case letters and numbers.\n"),
+				getdef_num ("PASS_MIN_LEN", 5), pass_max_len);
+		}
+	}
 
 	warned = 0;
 	for (i = getdef_num ("PASS_CHANGE_TRIES", 5); i > 0; i--) {
@@ -287,7 +319,7 @@ Please use a combination of upper and lower case letters and numbers.\n"), getde
 	/*
 	 * Encrypt the password, then wipe the cleartext password.
 	 */
-	cp = pw_encrypt (pass, crypt_make_salt ());
+	cp = pw_encrypt (pass, crypt_make_salt (NULL, NULL));
 	memzero (pass, sizeof pass);
 
 #ifdef HAVE_LIBCRACK_HIST
@@ -399,7 +431,7 @@ static void print_status (const struct passwd *pw)
 {
 	struct spwd *sp;
 
-	sp = getspnam (pw->pw_name);
+	sp = getspnam (pw->pw_name); /* local, no need for xgetspnam */
 	if (sp) {
 		printf ("%s %s %s %ld %ld %ld %ld\n",
 			pw->pw_name,
@@ -438,8 +470,16 @@ static char *update_crypt_pw (char *cp)
 	if (dflg)
 		cp = "";	/* XXX warning: const */
 
-	if (uflg && *cp == '!')
-		cp++;
+	if (uflg && *cp == '!') {
+		if (cp[1] == '\0') {
+			fprintf (stderr,
+				 _("%s: unlocking the user would result in a passwordless account.\n"
+				   "You should set a password with usermod -p to unlock this user account.\n"),
+				 Prog);
+		} else {
+			cp++;
+		}
+        }
 
 	if (lflg && *cp != '!') {
 		char *newpw = xmalloc (strlen (cp) + 2);
@@ -531,6 +571,15 @@ static void update_shadow (void)
 		nsp->sp_inact = (inact * DAY) / SCALE;
 	if (do_update_age)
 		nsp->sp_lstchg = time ((time_t *) 0) / SCALE;
+	if (lflg) {
+		/* Set the account expiry field to 1.
+		 * Some PAM implementation consider zero as a non expired
+		 * account.
+		 */
+		nsp->sp_expire = 1;
+	}
+	if (uflg)
+		nsp->sp_expire = -1;
 
 	/*
 	 * Force change on next login, like SunOS 4.x passwd -e or Solaris
@@ -645,7 +694,7 @@ int main (int argc, char **argv)
 			{"repository", required_argument, NULL, 'r'},
 			{"status", no_argument, NULL, 'S'},
 			{"unlock", no_argument, NULL, 'u'},
-			{"warning", required_argument, NULL, 'w'},
+			{"warndays", required_argument, NULL, 'w'},
 			{"maxdays", required_argument, NULL, 'x'},
 			{NULL, 0, NULL, '\0'}
 		};
@@ -786,7 +835,7 @@ int main (int argc, char **argv)
 		exit (E_NOPERM);
 	}
 
-	pw = getpwnam (name);
+	pw = xgetpwnam (name);
 	if (!pw) {
 		fprintf (stderr, _("%s: unknown user %s\n"), Prog, name);
 		exit (E_NOPERM);
@@ -826,7 +875,7 @@ int main (int argc, char **argv)
 	/*
 	 * The user name is valid, so let's get the shadow file entry.
 	 */
-	sp = getspnam (name);
+	sp = getspnam (name); /* !USE_PAM, no need for xgetspnam */
 	if (!sp)
 		sp = pwd_to_spwd (pw);
 
