@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: pwck.c,v 1.35 2006/05/07 17:44:39 kloczek Exp $"
+#ident "$Id: pwck.c 1397 2007-11-18 23:15:26Z nekral-guest $"
 
 #include <fcntl.h>
 #include <grp.h>
@@ -331,6 +331,7 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure the primary group exists
 		 */
+		/* local, no need for xgetgrgid */
 		if (!quiet && !getgrgid (pwd->pw_gid)) {
 
 			/*
@@ -368,7 +369,7 @@ int main (int argc, char **argv)
 				pwd->pw_name, pwd->pw_shell);
 			errors++;
 		}
-#ifdef SHADOWPWD
+
 		/*
 		 * Make sure this entry exists in the /etc/gshadow file.
 		 */
@@ -421,7 +422,6 @@ int main (int argc, char **argv)
 				}
 			}
 		}
-#endif
 	}
 
 	if (!is_shadow)
