@@ -47,7 +47,7 @@
 
 #include <config.h>
 
-#ident "$Id: su.c,v 1.74 2006/08/01 10:30:02 kloczek Exp $"
+#ident "$Id: su.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <getopt.h>
 #include <grp.h>
@@ -125,7 +125,7 @@ static int iswheel (const char *username)
 {
 	struct group *grp;
 
-	grp = getgrnam ("wheel");;
+	grp = getgrnam ("wheel"); /* !USE_PAM, no need for xgetgrnam */
 	if (!grp || !grp->gr_mem)
 		return 0;
 	return is_on_list (grp->gr_mem, username);
@@ -183,7 +183,12 @@ static void run_shell (const char *shellstr, char *args[], int doshell,
 
 	child = fork ();
 	if (child == 0) {	/* child shell */
-		pam_end (pamh, PAM_SUCCESS);
+		/*
+		 * PAM_DATA_SILENT is not supported by some modules, and
+		 * there is no strong need to clean up the process space's
+		 * memory since we will either call exec or exit.
+		pam_end (pamh, PAM_SUCCESS | PAM_DATA_SILENT);
+		 */
 
 		if (doshell)
 			(void) shell (shellstr, (char *) args[0], envp);
@@ -272,13 +277,13 @@ static void usage (void)
 	fprintf (stderr, _("Usage: su [options] [LOGIN]\n"
 			   "\n"
 			   "Options:\n"
-			   "  -c, --command COMMAND		pass COMMAND to the invoked shell\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -, -l, --login		make the shell a login shell\n"
+			   "  -c, --command COMMAND         pass COMMAND to the invoked shell\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -, -l, --login                make the shell a login shell\n"
 			   "  -m, -p,\n"
-			   "  --preserve-environment	do not reset environment variables, and keep\n"
-			   "				the same shell\n"
-			   "  -s, --shell SHELL		use SHELL instead of the default in passwd\n"
+			   "  --preserve-environment        do not reset environment variables, and\n"
+			   "                                keep the same shell\n"
+			   "  -s, --shell SHELL             use SHELL instead of the default in passwd\n"
 			   "\n"));
 	exit (E_USAGE);
 }
@@ -467,7 +472,7 @@ int main (int argc, char **argv)
 	 * Sort out the password of user calling su, in case needed later
 	 * -- chris
 	 */
-	if ((spwd = getspnam (oldname)))
+	if ((spwd = getspnam (oldname))) /* !USE_PAM, no need for xgetspnam */
 		pw->pw_passwd = spwd->sp_pwdp;
 	oldpass = xstrdup (pw->pw_passwd);
 #endif				/* SU_ACCESS */
@@ -502,7 +507,7 @@ int main (int argc, char **argv)
 	 * The password file entries for the user is gotten and the account
 	 * validated.
 	 */
-	if (!(pw = getpwnam (name))) {
+	if (!(pw = xgetpwnam (name))) {
 		(void) fprintf (stderr, _("Unknown id: %s\n"), name);
 		closelog ();
 		exit (1);
@@ -510,7 +515,7 @@ int main (int argc, char **argv)
 #ifndef USE_PAM
 	spwd = NULL;
 	if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0
-	    && (spwd = getspnam (name)))
+	    && (spwd = getspnam (name))) /* !USE_PAM, no need for xgetspnam */
 		pw->pw_passwd = spwd->sp_pwdp;
 #endif				/* !USE_PAM */
 	pwent = *pw;
@@ -691,8 +696,10 @@ int main (int argc, char **argv)
 			spwd = pwd_to_spwd (&pwent);
 
 		if (expire (&pwent, spwd)) {
+			/* !USE_PAM, no need for xgetpwnam */
 			struct passwd *pwd = getpwnam (name);
 
+			/* !USE_PAM, no need for xgetspnam */
 			spwd = getspnam (name);
 			if (pwd)
 				pwent = *pwd;
