@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: userdel.c,v 1.67 2006/08/15 17:25:58 kloczek Exp $"
+#ident "$Id: userdel.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -102,9 +102,10 @@ static void usage (void)
 	fprintf (stderr, _("Usage: userdel [options] LOGIN\n"
 			   "\n"
 			   "Options:\n"
-			   "  -f, --force			force removal of files, even if not owned by user\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -r, --remove			remove home directory and mail spool\n"
+			   "  -f, --force                   force removal of files,\n"
+			   "                                even if not owned by user\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -r, --remove                  remove home directory and mail spool\n"
 			   "\n"));
 	exit (E_USAGE);
 }
@@ -150,12 +151,17 @@ static void update_groups (void)
 		 */
 		ngrp = __gr_dup (grp);
 		if (!ngrp) {
+			fprintf (stderr,
+				 _("%s: Out of memory. Cannot update the group database.\n"),
+				 Prog);
 			exit (13);	/* XXX */
 		}
 		ngrp->gr_mem = del_list (ngrp->gr_mem, user_name);
-		if (!gr_update (ngrp))
+		if (!gr_update (ngrp)) {
 			fprintf (stderr,
 				 _("%s: error updating group entry\n"), Prog);
+			exit (E_GRP_UPDATE);
+		}
 
 		/*
 		 * Update the DBM group file with the new entry as well.
@@ -174,7 +180,7 @@ static void update_groups (void)
 	 * now if they have a group with the same name as their
 	 * user name, with no members, we delete it.
 	 */
-	grp = getgrnam (user_name);
+	grp = xgetgrnam (user_name);
 	if (grp && getdef_bool ("USERGROUPS_ENAB")
 	    && (grp->gr_mem[0] == NULL)) {
 
@@ -243,6 +249,9 @@ static void update_groups (void)
 
 		nsgrp = __sgr_dup (sgrp);
 		if (!nsgrp) {
+			fprintf (stderr,
+				 _("%s: Out of memory. Cannot update the shadow group database.\n"),
+				 Prog);
 			exit (13);	/* XXX */
 		}
 
@@ -252,9 +261,11 @@ static void update_groups (void)
 		if (was_admin)
 			nsgrp->sg_adm = del_list (nsgrp->sg_adm, user_name);
 
-		if (!sgr_update (nsgrp))
+		if (!sgr_update (nsgrp)) {
 			fprintf (stderr,
-				 _("%s: error updating group entry\n"), Prog);
+				 _("%s: error updating shadow group entry\n"), Prog);
+			exit (E_GRP_UPDATE);
+		}
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 			      "deleting user from shadow group", user_name,
@@ -411,12 +422,16 @@ static void open_files (void)
  */
 static void update_user (void)
 {
-	if (!pw_remove (user_name))
+	if (!pw_remove (user_name)) {
 		fprintf (stderr,
 			 _("%s: error deleting password entry\n"), Prog);
-	if (is_shadow_pwd && !spw_remove (user_name))
+		fail_exit (E_PW_UPDATE);
+	}
+	if (is_shadow_pwd && !spw_remove (user_name)) {
 		fprintf (stderr,
 			 _("%s: error deleting shadow password entry\n"), Prog);
+		fail_exit (E_PW_UPDATE);
+	}
 #ifdef WITH_AUDIT
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting user entries",
 		      user_name, user_id, 1);
@@ -528,7 +543,8 @@ static void user_cancel (const char *user)
 #ifdef EXTRA_CHECK_HOME_DIR
 static int path_prefix (const char *s1, const char *s2)
 {
-	return (strncmp (s2, s1, strlen (s1)) == 0);
+	return (strncmp (s2, s1, strlen (s1)) == 0 &&
+		(s2[strlen (s1)] == '\0' || s2[strlen (s1)] == '/'));
 }
 #endif
 
@@ -593,12 +609,10 @@ static void remove_mailbox (void)
  */
 int main (int argc, char **argv)
 {
-	struct passwd *pwd;
 	int errors = 0;
 
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -648,13 +662,19 @@ int main (int argc, char **argv)
 
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
 
-	if (retval == PAM_SUCCESS)
-		retval = pam_start ("userdel", pampw->pw_name, &conv, &pamh);
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
+
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("userdel", pampw->pw_name,
+					    &conv, &pamh);
+		}
+	}
 
 	if (retval == PAM_SUCCESS) {
 		retval = pam_authenticate (pamh, 0);
@@ -683,14 +703,20 @@ int main (int argc, char **argv)
 	 * Start with a quick check to see if the user exists.
 	 */
 	user_name = argv[argc - 1];
-	if (!(pwd = getpwnam (user_name))) {
-		fprintf (stderr, _("%s: user %s does not exist\n"),
-			 Prog, user_name);
+	{
+		struct passwd *pwd;
+		/* local, no need for xgetpwnam */
+		if (!(pwd = getpwnam (user_name))) {
+			fprintf (stderr, _("%s: user %s does not exist\n"),
+				 Prog, user_name);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "deleting user not found", user_name, -1, 0);
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "deleting user not found", user_name, -1, 0);
 #endif
-		exit (E_NOTFOUND);
+			exit (E_NOTFOUND);
+		}
+		user_id = pwd->pw_uid;
+		user_home = xstrdup (pwd->pw_dir);
 	}
 #ifdef	USE_NIS
 
@@ -712,8 +738,6 @@ int main (int argc, char **argv)
 		exit (E_NOTFOUND);
 	}
 #endif
-	user_id = pwd->pw_uid;
-	user_home = xstrdup (pwd->pw_dir);
 	/*
 	 * Check to make certain the user isn't logged in.
 	 */
@@ -740,6 +764,7 @@ int main (int argc, char **argv)
 #ifdef EXTRA_CHECK_HOME_DIR
 	/* This may be slow, the above should be good enough. */
 	if (rflg && !fflg) {
+		struct passwd *pwd;
 		/*
 		 * For safety, refuse to remove the home directory if it
 		 * would result in removing some other user's home
