@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: usermod.c,v 1.74 2007/01/16 12:51:50 kloczek Exp $"
+#ident "$Id: usermod.c 1460 2007-11-24 22:41:24Z nekral-guest $"
 
 #include <ctype.h>
 #include <errno.h>
@@ -105,7 +105,7 @@ static long user_newinactive;	/* Audit */
 static char *Prog;
 
 static int
- aflg = 0,			/* append to existing secondary group set */
+    aflg = 0,			/* append to existing secondary group set */
     cflg = 0,			/* new comment (GECOS) field */
     dflg = 0,			/* new home directory */
     eflg = 0,			/* days since 1970-01-01 when account becomes expired */
@@ -127,6 +127,13 @@ static int is_shadow_pwd;
 static int is_shadow_grp;
 #endif
 
+static int pw_locked  = 0;
+static int spw_locked = 0;
+static int gr_locked  = 0;
+#ifdef SHADOWGRP
+static int sgr_locked = 0;
+#endif
+
 
 /* local function prototypes */
 static int get_groups (char *);
@@ -135,12 +142,12 @@ static void new_pwent (struct passwd *);
 
 static void new_spent (struct spwd *);
 static void fail_exit (int);
-static int update_group (void);
+static void update_group (void);
 
 #ifdef SHADOWGRP
-static int update_gshadow (void);
+static void update_gshadow (void);
 #endif
-static int grp_update (void);
+static void grp_update (void);
 
 static long get_number (const char *);
 static uid_t get_id (const char *);
@@ -167,8 +174,8 @@ static struct group *getgr_nam_gid (const char *grname)
 
 	val = strtol (grname, &errptr, 10);
 	if (*grname != '\0' && *errptr == '\0' && errno != ERANGE && val >= 0)
-		return getgrgid (val);
-	return getgrnam (grname);
+		return xgetgrgid (val);
+	return xgetgrnam (grname);
 }
 
 /*
@@ -275,26 +282,26 @@ static void usage (void)
 	fprintf (stderr, _("Usage: usermod [options] LOGIN\n"
 			   "\n"
 			   "Options:\n"
-			   "  -c, --comment COMMENT		new value of the GECOS field\n"
-			   "  -d, --home HOME_DIR		new home directory for the user account\n"
-			   "  -e, --expiredate EXPIRE_DATE	set account expiration date to EXPIRE_DATE\n"
-			   "  -f, --inactive INACTIVE	set password inactive after expiration\n"
-			   "				to INACTIVE\n"
-			   "  -g, --gid GROUP		force use GROUP as new primary group\n"
-			   "  -G, --groups GROUPS		new list of supplementary GROUPS\n"
-			   "  -a, --append		append the user to the supplemental GROUPS\n"
-			   "				mentioned by the -G option without removing\n"
-			   "				him/her from other groups\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -l, --login NEW_LOGIN		new value of the login name\n"
-			   "  -L, --lock			lock the user account\n"
-			   "  -m, --move-home		move contents of the home directory to the new\n"
-			   "				location (use only with -d)\n"
-			   "  -o, --non-unique		allow using duplicate (non-unique) UID\n"
-			   "  -p, --password PASSWORD	use encrypted password for the new password\n"
-			   "  -s, --shell SHELL		new login shell for the user account\n"
-			   "  -u, --uid UID			new UID for the user account\n"
-			   "  -U, --unlock			unlock the user account\n"
+			   "  -c, --comment COMMENT         new value of the GECOS field\n"
+			   "  -d, --home HOME_DIR           new home directory for the user account\n"
+			   "  -e, --expiredate EXPIRE_DATE  set account expiration date to EXPIRE_DATE\n"
+			   "  -f, --inactive INACTIVE       set password inactive after expiration\n"
+			   "                                to INACTIVE\n"
+			   "  -g, --gid GROUP               force use GROUP as new primary group\n"
+			   "  -G, --groups GROUPS           new list of supplementary GROUPS\n"
+			   "  -a, --append                  append the user to the supplemental GROUPS\n"
+			   "                                mentioned by the -G option without removing\n"
+			   "                                him/her from other groups\n"
+			   "  -h, --help                    display this help message and exit\n"
+			   "  -l, --login NEW_LOGIN         new value of the login name\n"
+			   "  -L, --lock                    lock the user account\n"
+			   "  -m, --move-home               move contents of the home directory to the\n"
+			   "                                new location (use only with -d)\n"
+			   "  -o, --non-unique              allow using duplicate (non-unique) UID\n"
+			   "  -p, --password PASSWORD       use encrypted password for the new password\n"
+			   "  -s, --shell SHELL             new login shell for the user account\n"
+			   "  -u, --uid UID                 new UID for the user account\n"
+			   "  -U, --unlock                  unlock the user account\n"
 			   "\n"));
 	exit (E_USAGE);
 }
@@ -319,6 +326,14 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 	} else if (Uflg && pw_pass[0] == '!') {
 		char *s;
 
+		if (pw_pass[1] == '\0') {
+			fprintf (stderr,
+				 _("%s: unlocking the user would result in a passwordless account.\n"
+				   "You should set a password with usermod -p to unlock this user account.\n"),
+				 Prog);
+			return pw_pass;
+		}
+
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating password",
 			      user_newname, user_newid, 0);
@@ -514,14 +529,17 @@ static void new_spent (struct spwd *spent)
  */
 static void fail_exit (int code)
 {
-	(void) gr_unlock ();
+	if (gr_locked)
+		gr_unlock ();
 #ifdef	SHADOWGRP
-	if (is_shadow_grp)
+	if (sgr_locked)
 		sgr_unlock ();
 #endif
-	if (is_shadow_pwd)
+	if (spw_locked)
 		spw_unlock ();
-	(void) pw_unlock ();
+	if (pw_locked)
+		pw_unlock ();
+
 #ifdef WITH_AUDIT
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying account",
 		      user_name, -1, 0);
@@ -530,7 +548,7 @@ static void fail_exit (int code)
 }
 
 
-static int update_group (void)
+static void update_group (void)
 {
 	int is_member;
 	int was_member;
@@ -538,22 +556,6 @@ static int update_group (void)
 	const struct group *grp;
 	struct group *ngrp;
 
-	/*
-	 * Lock and open the group file. This will load all of the group
-	 * entries.
-	 */
-	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: error locking group file\n"), Prog);
-		SYSLOG ((LOG_ERR, "error locking group file"));
-		return -1;
-	}
-	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: error opening group file\n"), Prog);
-		SYSLOG ((LOG_ERR, "error opening group file"));
-		gr_unlock ();
-		return -1;
-	}
-
 	changed = 0;
 
 	/*
@@ -574,10 +576,9 @@ static int update_group (void)
 		ngrp = __gr_dup (grp);
 		if (!ngrp) {
 			fprintf (stderr,
-				 _("%s: out of memory in update_group\n"),
+				 _("%s: Out of memory. Cannot update the group database.\n"),
 				 Prog);
-			gr_unlock ();
-			return -1;
+			fail_exit (E_GRP_UPDATE);
 		}
 
 		if (was_member && (!Gflg || is_member)) {
@@ -627,23 +628,14 @@ static int update_group (void)
 		if (!gr_update (ngrp)) {
 			fprintf (stderr,
 				 _("%s: error adding new group entry\n"), Prog);
-			SYSLOG ((LOG_ERR, "error adding group entry"));
-			gr_unlock ();
-			return -1;
+			SYSLOG ((LOG_ERR, "error adding new group entry"));
+			fail_exit (E_GRP_UPDATE);
 		}
 	}
-
-	if (!gr_close ()) {
-		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
-		gr_unlock ();
-		return -1;
-	}
-	gr_unlock ();
-	return 0;
 }
 
 #ifdef SHADOWGRP
-static int update_gshadow (void)
+static void update_gshadow (void)
 {
 	int is_member;
 	int was_member;
@@ -652,20 +644,6 @@ static int update_gshadow (void)
 	const struct sgrp *sgrp;
 	struct sgrp *nsgrp;
 
-	if (!sgr_lock ()) {
-		fprintf (stderr,
-			 _("%s: error locking shadow group file\n"), Prog);
-		SYSLOG ((LOG_ERR, "error locking shadow group file"));
-		return -1;
-	}
-	if (!sgr_open (O_RDWR)) {
-		fprintf (stderr,
-			 _("%s: error opening shadow group file\n"), Prog);
-		SYSLOG ((LOG_ERR, "error opening shadow group file"));
-		sgr_unlock ();
-		return -1;
-	}
-
 	changed = 0;
 
 	/*
@@ -696,11 +674,9 @@ static int update_gshadow (void)
 		nsgrp = __sgr_dup (sgrp);
 		if (!nsgrp) {
 			fprintf (stderr,
-				 _
-				 ("%s: out of memory in update_gshadow\n"),
+				 _("%s: Out of memory. Cannot update the shadow group database.\n"),
 				 Prog);
-			sgr_unlock ();
-			return -1;
+			fail_exit (E_GRP_UPDATE);
 		}
 
 		if (was_admin && lflg) {
@@ -768,21 +744,11 @@ static int update_gshadow (void)
 		 */
 		if (!sgr_update (nsgrp)) {
 			fprintf (stderr,
-				 _("%s: error adding new group entry\n"), Prog);
+				 _("%s: error adding new shadow group entry\n"), Prog);
 			SYSLOG ((LOG_ERR, "error adding shadow group entry"));
-			sgr_unlock ();
-			return -1;
+			fail_exit (E_GRP_UPDATE);
 		}
 	}
-
-	if (!sgr_close ()) {
-		fprintf (stderr,
-			 _("%s: cannot rewrite shadow group file\n"), Prog);
-		sgr_unlock ();
-		return -1;
-	}
-	sgr_unlock ();
-	return 0;
 }
 #endif				/* SHADOWGRP */
 
@@ -792,16 +758,13 @@ static int update_gshadow (void)
  *	grp_update() takes the secondary group set given in user_groups and
  *	adds the user to each group given by that set.
  */
-static int grp_update (void)
+static void grp_update (void)
 {
-	int ret;
-
-	ret = update_group ();
+	update_group ();
 #ifdef SHADOWGRP
-	if (!ret && is_shadow_grp)
-		ret = update_gshadow ();
+	if (is_shadow_grp)
+		update_gshadow ();
 #endif
-	return ret;
 }
 
 static long get_number (const char *numstr)
@@ -842,26 +805,28 @@ static uid_t get_id (const char *uidstr)
 static void process_flags (int argc, char **argv)
 {
 	const struct group *grp;
-	const struct passwd *pwd;
 
-	const struct spwd *spwd = NULL;
 	int anyflag = 0;
 
 	if (argc == 1 || argv[argc - 1][0] == '-')
 		usage ();
 
-	if (!(pwd = getpwnam (argv[argc - 1]))) {
-		fprintf (stderr, _("%s: user %s does not exist\n"),
-			 Prog, argv[argc - 1]);
-		exit (E_NOTFOUND);
-	}
+	{
+		const struct passwd *pwd;
+		/* local, no need for xgetpwnam */
+		if (!(pwd = getpwnam (argv[argc - 1]))) {
+			fprintf (stderr, _("%s: user %s does not exist\n"),
+				 Prog, argv[argc - 1]);
+			exit (E_NOTFOUND);
+		}
 
-	user_name = argv[argc - 1];
-	user_id = pwd->pw_uid;
-	user_gid = pwd->pw_gid;
-	user_comment = xstrdup (pwd->pw_gecos);
-	user_home = xstrdup (pwd->pw_dir);
-	user_shell = xstrdup (pwd->pw_shell);
+		user_name = argv[argc - 1];
+		user_id = pwd->pw_uid;
+		user_gid = pwd->pw_gid;
+		user_comment = xstrdup (pwd->pw_gecos);
+		user_home = xstrdup (pwd->pw_dir);
+		user_shell = xstrdup (pwd->pw_shell);
+	}
 #ifdef WITH_AUDIT
 	user_newname = user_name;
 	user_newid = user_id;
@@ -891,13 +856,17 @@ static void process_flags (int argc, char **argv)
 	}
 #endif
 
-	if (is_shadow_pwd && (spwd = getspnam (user_name))) {
-		user_expire = spwd->sp_expire;
-		user_inactive = spwd->sp_inact;
+	{
+		const struct spwd *spwd = NULL;
+		/* local, no need for xgetspnam */
+		if (is_shadow_pwd && (spwd = getspnam (user_name))) {
+			user_expire = spwd->sp_expire;
+			user_inactive = spwd->sp_inact;
 #ifdef WITH_AUDIT
-		user_newexpire = user_expire;
-		user_newinactive = user_inactive;
+			user_newexpire = user_expire;
+			user_newinactive = user_inactive;
 #endif
+		}
 	}
 
 	{
@@ -960,7 +929,7 @@ static void process_flags (int argc, char **argv)
 #ifdef WITH_AUDIT
 					user_newexpire = strtoday (optarg);
 					if (user_newexpire == -1) {
-#else
+#else /* } */
 					user_expire = strtoday (optarg);
 					if (user_expire == -1) {
 #endif
@@ -1026,27 +995,15 @@ static void process_flags (int argc, char **argv)
 				user_newname = optarg;
 				break;
 			case 'L':
-				if (Uflg || pflg)
-					usage ();
-
 				Lflg++;
 				break;
 			case 'm':
-				if (!dflg)
-					usage ();
-
 				mflg++;
 				break;
 			case 'o':
-				if (!uflg)
-					usage ();
-
 				oflg++;
 				break;
 			case 'p':
-				if (Lflg || Uflg)
-					usage ();
-
 				user_pass = optarg;
 				pflg++;
 				break;
@@ -1069,9 +1026,6 @@ static void process_flags (int argc, char **argv)
 				uflg++;
 				break;
 			case 'U':
-				if (Lflg && pflg)
-					usage ();
-
 				Uflg++;
 				break;
 			default:
@@ -1098,23 +1052,49 @@ static void process_flags (int argc, char **argv)
 
 	if (aflg && (!Gflg)) {
 		fprintf (stderr,
-			 _("%s: -a flag is ONLY allowed with the -G flag\n"),
+			 _("%s: %s flag is ONLY allowed with the %s flag\n"),
+			 Prog, "-a", "-G");
+		usage ();
+		exit (E_USAGE);
+	}
+
+	if ((Lflg && (pflg || Uflg)) || (pflg && Uflg)) {
+		fprintf (stderr,
+			 _("%s: the -L, -p, and -U flags are exclusive\n"),
 			 Prog);
 		usage ();
 		exit (E_USAGE);
 	}
 
+	if (oflg && !uflg) {
+		fprintf (stderr,
+			 _("%s: %s flag is ONLY allowed with the %s flag\n"),
+			 Prog, "-o", "-u");
+		usage ();
+		exit (E_USAGE);
+	}
+
+	if (mflg && !dflg) {
+		fprintf (stderr,
+			 _("%s: %s flag is ONLY allowed with the %s flag\n"),
+			 Prog, "-m", "-d");
+		usage ();
+		exit (E_USAGE);
+	}
+
 	if (dflg && strcmp (user_home, user_newhome) == 0)
 		dflg = mflg = 0;
 
 	if (uflg && user_id == user_newid)
 		uflg = oflg = 0;
 
+	/* local, no need for xgetpwnam */
 	if (lflg && getpwnam (user_newname)) {
 		fprintf (stderr, _("%s: user %s exists\n"), Prog, user_newname);
 		exit (E_NAME_IN_USE);
 	}
 
+	/* local, no need for xgetpwuid */
 	if (uflg && !oflg && getpwuid (user_newid)) {
 		fprintf (stderr, _("%s: uid %lu is not unique\n"),
 			 Prog, (unsigned long) user_newid);
@@ -1139,9 +1119,34 @@ static void close_files (void)
 			 _("%s: cannot rewrite shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
+
+	if (Gflg || lflg) {
+		if (!gr_close ()) {
+			fprintf (stderr, _("%s: cannot rewrite group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+#ifdef SHADOWGRP
+		if (is_shadow_grp && !sgr_close ()) {
+			fprintf (stderr,
+				 _("%s: cannot rewrite shadow group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		if (is_shadow_grp)
+			sgr_unlock ();
+#endif
+		gr_unlock ();
+	}
+
 	if (is_shadow_pwd)
 		spw_unlock ();
-	(void) pw_unlock ();
+	pw_unlock ();
+
+	pw_locked = 0;
+	spw_locked = 0;
+	gr_locked = 0;
+	sgr_locked = 0;
 
 	/*
 	 * Close the DBM and/or flat files
@@ -1163,8 +1168,9 @@ static void open_files (void)
 {
 	if (!pw_lock ()) {
 		fprintf (stderr, _("%s: unable to lock password file\n"), Prog);
-		exit (E_PW_UPDATE);
+		fail_exit (E_PW_UPDATE);
 	}
+	pw_locked = 1;
 	if (!pw_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
@@ -1174,11 +1180,45 @@ static void open_files (void)
 			 _("%s: cannot lock shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
+	spw_locked = 1;
 	if (is_shadow_pwd && !spw_open (O_RDWR)) {
 		fprintf (stderr,
 			 _("%s: cannot open shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
+
+	if (Gflg || lflg) {
+		/*
+		 * Lock and open the group file. This will load all of the
+		 * group entries.
+		 */
+		if (!gr_lock ()) {
+			fprintf (stderr, _("%s: error locking group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		gr_locked = 1;
+		if (!gr_open (O_RDWR)) {
+			fprintf (stderr, _("%s: error opening group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+#ifdef SHADOWGRP
+		if (is_shadow_grp && !sgr_lock ()) {
+			fprintf (stderr,
+				 _("%s: error locking shadow group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		sgr_locked = 1;
+		if (is_shadow_grp && !sgr_open (O_RDWR)) {
+			fprintf (stderr,
+				 _("%s: error opening shadow group file\n"),
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+#endif
+	}
 }
 
 /*
@@ -1455,11 +1495,8 @@ static void move_mailbox (void)
  */
 int main (int argc, char **argv)
 {
-	int grp_err = 0;
-
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -1492,13 +1529,17 @@ int main (int argc, char **argv)
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("usermod", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("usermod", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -1527,14 +1568,13 @@ int main (int argc, char **argv)
 	 */
 	open_files ();
 	usr_update ();
+	if (Gflg || lflg)
+		grp_update ();
 	close_files ();
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
 
-	if (Gflg || lflg)
-		grp_err = grp_update ();
-
 	if (mflg)
 		move_home ();
 
@@ -1555,9 +1595,6 @@ int main (int argc, char **argv)
 			    user_gid, gflg ? user_newgid : user_gid);
 	}
 
-	if (grp_err)
-		exit (E_GRP_UPDATE);
-
 #ifdef USE_PAM
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
